package com.example.apptics_flutter

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.util.Log
import com.zoho.apptics.analytics.AppticsAnalytics
import com.zoho.apptics.analytics.AppticsAnalytics.showReviewTrackingSettingsPopup
import com.zoho.apptics.analytics.AppticsEvents
import com.zoho.apptics.analytics.AppticsScreenTracker
import com.zoho.apptics.appupdates.AppticsInAppUpdates
import com.zoho.apptics.common.Apptics
import com.zoho.apptics.common.AppticsSettings
import com.zoho.apptics.common.AppticsSettings.setLocale
import com.zoho.apptics.common.AppticsSettings.setTrackingStatus
import com.zoho.apptics.common.AppticsTrackingState
import com.zoho.apptics.common.AppticsUser
import com.zoho.apptics.crash.AppticsCrashTracker
import com.zoho.apptics.feedback.AppticsFeedback
import com.zoho.apptics.rateus.AppticsInAppRatings
import com.zoho.apptics.rateus.AppticsInAppRatings.showPopupIfCriteriaFulfilled
import com.zoho.apptics.remoteconfig.AppticsRemoteConfig
import io.flutter.embedding.android.FlutterActivity
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import org.json.JSONObject
import java.util.Locale

const val TAG = "flutter_apptics_debug"

/** AppticsFlutterPlugin */
class AppticsFlutterPlugin : FlutterPlugin, MethodCallHandler,
    PluginRegistry.ActivityResultListener, ActivityAware {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity

    private lateinit var channel: MethodChannel
    private var currentActivity: Activity? = null
    private val FILE_NAME = "apptics_flutter_sdk"
    private val LAST_API_CALL_TIME = "lastApiCallTime"


    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "apptics_flutter")
        channel.setMethodCallHandler(this)
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    // Start
    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        currentActivity = binding.activity
        try {
            Apptics.init(binding.activity)
            val mapping = Class.forName("com.zoho.apptics.analytics.AppticsEventsIdMapping")
            mapping.getMethod("init").invoke(mapping.newInstance())
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    override fun onDetachedFromActivityForConfigChanges() {}
    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {}

    override fun onDetachedFromActivity() {
        currentActivity = null
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "getPlatformVersion" -> {
                getPlatformVersion(result)
            }

            "setUser" -> {
                setCurrentUser(call, result)
            }

            "isUserLoggedIn" -> {
                isUserLoggedIn(result)
            }

            "setDefaultLanguage" -> {
                setDefaultLanguage(call, result)
            }

            "setTrackingState" -> {
                setTrackingStatus(call, result)
            }

            "getTrackingState" -> {
                getTrackingStatus(result)
            }

            "openPrivacySettings" -> {
                openSettings(result)
            }

            "addEvent" -> {
                addEvent(call, result)
            }

            "screenAttached" -> {
                trackScreenAttached(call, result)
            }

            "screenDetached" -> {
                trackScreenDetached(call, result)
            }

            "showLastSessionCrashedPopup" -> {
                showLastSessionCrashedPopup(result)
            }

            "presentPrivacyReviewPopup" -> {
                showPrivacyReviewPopup(result)
            }

            "setCrashCustomProperty" -> {
                setCustomProperties(call, result)
            }

            "checkAndUpdateAlert" -> {
                checkAndUpdateAlert(result)
            }

            "onClickUpdate" -> {
                onClickUpdate(result)
            }

            "onClickReminder" -> {
                onClickReminder(result)
            }

            "onClickIgnore" -> {
                onClickIgnore(result)
            }

            "onClickNonSupportAlert" -> {
                onClickNonSupportAlert(result)
            }

            "onSendImpressionStatus" -> {
                onSendImpressionStatus(call, result)
            }

            "openFeedbackScreen" -> {
                openFeedbackScreen(result)
            }

            // In-app rating
            "checkForRatingPop" -> {
                checkForRatingPop(result)
            }

            "isAppticsFeedbackModuleAvailable" -> {
                isAppticsFeedbackModuleAvailable(result)
            }

            "openPlayStore" -> {
                openPlayStore(result)
            }

            "sentRatingStats" -> {
                sentRatingStats(call, result)
            }

            "setDisableIfNotInstalledFromPlayStore" -> {
                setDisableIfNotInstalledFromPlayStore(call, result)
            }

            "setShowStoreAlertOnFulFillingCriteria" -> {
                setShowStoreAlertOnFulFillingCriteria(call, result)
            }

            "setDisableAutoPromptOnFulFillingCriteria" -> {
                setDisableAutoPromptOnFulFillingCriteria(call, result)
            }

            "setMaximumTimesPop" -> {
                setMaximumTimesPop(call, result)
            }

            "setDaysBeforeShowingPopupAgain" -> {
                setDaysBeforeShowingPopupAgain(call, result)
            }

            "getStringValue" -> {
                fetchValue(call, result)
            }

            "setCustomConditionValue" -> {
                setCustomConditionValue(call, result)
            }

            "updateRatingShown" -> {
                markPopupShown(result)
            }

            else -> {
                result.notImplemented()
            }
        }
    }


    // Remote Config

    private fun fetchValue(call: MethodCall, result: Result) {
        val currentTime = System.currentTimeMillis()
        val arg = call.arguments as Map<String, Any>
        val key = arg["key"] as String
        val coldFetch = arg["coldFetch"] as Boolean
        val fallbackWithOfflineValue = arg["fallbackWithOfflineValue"] as Boolean
        CoroutineScope(Dispatchers.IO).launch {
            if ((coldFetch && currentActivity != null)) {
                val lastApiCallTime = currentActivity!!.getPre()!!.getLong(LAST_API_CALL_TIME, 0L)
                if (currentTime - lastApiCallTime >= 10000) {
                    val responseStringValues = AppticsRemoteConfig.fetchValue(
                        key,
                        coldFetch = true,
                        fallbackWithOfflineValue = false
                    )
                    currentActivity!!.getPre()!!.edit().putLong(LAST_API_CALL_TIME, currentTime)
                        .apply()
                    responseStringValues?.let {
                        result.success(it)
                    } ?: result.success(null)
                } else if (fallbackWithOfflineValue) {
                    val responseStringValues =
                        AppticsRemoteConfig.fetchValue(key, false, fallbackWithOfflineValue = true)
                    responseStringValues?.let {
                        result.success(it)
                    } ?: result.success(null)
                } else {
                    result.success(null)
                }
            } else {
                val responseStringValues = AppticsRemoteConfig.fetchValue(
                    key,
                    fallbackWithOfflineValue = fallbackWithOfflineValue
                )
                responseStringValues?.let {
                    result.success(it)
                } ?: result.success(null)
            }
        }
    }

    private fun setCustomConditionValue(call: MethodCall, result: Result) {
        // {'key': key, 'value': value}
        val arg = call.arguments as? Map<String, String>
        val conditionKey = arg?.get("key")
        val value = arg?.get("value")
        if (conditionKey != null && value != null) {
            AppticsRemoteConfig.setCustomConditionValue(conditionKey, value)
            result.success(true)
        } else {
            result.success(false)
        }
    }


    // In-app Ratings


    private fun setDaysBeforeShowingPopupAgain(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, Int>
        val dayCount = arg["dayCount"]
        dayCount?.let {
            AppticsInAppRatings.daysBeforeShowingPopupAgain = it
            result.success(it)
        } ?: result.success(10)
    }

    private fun setMaximumTimesPop(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, Int>
        val maxTime = arg["maxTime"]
        maxTime?.let {
            AppticsInAppRatings.maxTimesToShowPopup = it
            result.success(it)
        } ?: result.success(3)

    }

    /**
     * Disable showing in-app ratings pop-up automatically on satisfying criterion.
     * @see showPopupIfCriteriaFulfilled
     */
    private fun setDisableAutoPromptOnFulFillingCriteria(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, Boolean>
        val isDisable = arg["isDisable"]
        isDisable?.let {
            AppticsInAppRatings.disableAutoPromptOnFulFillingCriteria = it
            result.success(it)
        } ?: result.success(false)
    }

    /**
     * To invoke play-core in-app ratings SDK on satisfying criterion.
     */
    private fun setShowStoreAlertOnFulFillingCriteria(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, Boolean>
        val isShow = arg["isShow"]
        isShow?.let {
            AppticsInAppRatings.showAndroidPlayCoreAlertOnFulFillingCriteria = it
            result.success(it)
        } ?: result.success(false)
    }

    /**
     * Flag to disable ratings if the app is not installed from playstore.
     */
    private fun setDisableIfNotInstalledFromPlayStore(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, Boolean>
        val ifNotInstalledFromPlayStore = arg["ifNotInstalledFromPlayStore"]
        ifNotInstalledFromPlayStore?.let {
            AppticsInAppRatings.disableIfNotInstalledFromPlayStore = it
            result.success(it)
        } ?: result.success(false)
    }


    private fun sentRatingStats(call: MethodCall, result: Result) {


        val arg = call.arguments as Map<String, Any>
        //{'criteriaId': criteriaId, 'popupAction': popupAction.value}
        val criteriaId = arg["criteriaId"] as Long
        val popupActionArg = arg["popupAction"] as Int
        var popupAction: AppticsInAppRatings.PopupAction? = null
        for (pop in AppticsInAppRatings.PopupAction.values()) {
            if (pop.value == popupActionArg) {
                popupAction = pop
            }
        }
        if (popupAction != null) {
            AppticsInAppRatings.sendStats(
                criteriaId,
                popupAction,
                AppticsInAppRatings.PopupSource.DYNAMIC
            )
            result.success(true)
        } else
            result.success(false)

    }

    private fun markPopupShown(result: Result) {
        AppticsInAppRatings.markPopupShown()
        result.success(true)
    }

    private fun openPlayStore(result: Result) {
        currentActivity?.let { activity ->
            AppticsInAppRatings.openStore(activity)
            result.success(true)
        } ?: result.success(false)
    }

    private fun checkForRatingPop(result: Result) {
        AppticsInAppRatings.flutterRateUsEnabled = true
        AppticsInAppRatings.flutterUiCallBack = { criteriaId ->
            if (criteriaId != 0L)
                result.success(criteriaId)
            else
                result.success(0)
        }
    }

    private fun isAppticsFeedbackModuleAvailable(result: Result) {
        result.success(AppticsInAppRatings.isAppticsFeedbackModuleAvailable())
    }

    private fun openFeedbackScreen(result: Result) {
        currentActivity?.let { activity ->
            AppticsFeedback.openFeedback(activity, "2")
            result.success(true)
        } ?: result.success(false)
    }


    private fun setDefaultLanguage(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val language = arg["lang"]
        val array = language?.split(Regex("[-_]"))
        val locale: Locale? = array?.let { list ->
            when (list.size) {
                2 -> Locale(list[0], list[1])
                3 -> Locale(list[0], list[1], list[2])
                else -> {
                    Locale(list[0])
                }
            }
        }
        result.success(locale?.let { setLocale(it).toString() })
    }

    private fun getPlatformVersion(result: Result) {
        result.success("Android ${android.os.Build.VERSION.RELEASE}")

    }

    private fun setCustomProperties(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, Map<String, String>>
        val json = arg["properties"]
        if (json == null) {
            result.success(false)
        } else {
            AppticsCrashTracker.customProperties = JSONObject(json)
            Log.d(TAG, "setCustomProperties: ${AppticsCrashTracker.customProperties}")
            result.success(true)
        }
    }

    private fun showPrivacyReviewPopup(result: Result) {
        currentActivity?.let {
            showReviewTrackingSettingsPopup(it)
            result.success(true)
        }
    }

    private fun showLastSessionCrashedPopup(result: Result) {
        currentActivity?.let {
            result.success(AppticsCrashTracker.showLastSessionCrashedPopup(it, true).toString())
        } ?: result.success(false)
    }

    private fun trackScreenAttached(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val screenName: String = arg["screen_name"] ?: ""
        println(screenName)
        AppticsScreenTracker.inScreen(screenName)
        result.success(true)
    }

    private fun trackScreenDetached(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val screenName: String = arg["screen_name"] ?: ""
        println(screenName)
        AppticsScreenTracker.outScreen(screenName)
        result.success(true)
    }


    private fun addEvent(call: MethodCall, result: Result) {
        val eventArg = call.arguments as Map<*, *>
        val eventName = eventArg["event"].toString()
        val groupName = eventArg["group"].toString()
        val customPropertiesMap = eventArg["properties"] as? HashMap<String, Any>
        val customProperties: JSONObject? = customPropertiesMap?.let {
            JSONObject().apply {
                customPropertiesMap.entries.forEach {
                    put(it.key, it.value)
                }
            }
        }
        AppticsEvents.addEvent(eventName, groupName, customProperties)
        result.success(true)
    }

    private fun openSettings(result: Result) {
        if (currentActivity != null) {
            AppticsAnalytics.openSettings(currentActivity!!)
            result.success(true)
        } else {
            println("Null ${this.currentActivity}")
            result.success(false)
        }
    }

    private fun getTrackingStatus(result: Result) {
        result.success(AppticsSettings.getTrackingStatus().value)
    }

    private fun setTrackingStatus(call: MethodCall, result: Result) {
        val trackingState = call.arguments as Map<String, Int>
        val status = trackingState["status"]
        for (state in AppticsTrackingState.values()) {
            if (state.value == status) {
                result.success(setTrackingStatus(state))
            }
        }
    }

    private fun isUserLoggedIn(result: Result) {
        val isUserLoggedIn = AppticsUser.getCurrentUser() != null
        result.success(isUserLoggedIn)
    }

    private fun setCurrentUser(call: MethodCall, result: Result) {
        val userIdArg = call.arguments as Map<String, String>
        val userId: String? = userIdArg["userId"]
        result.success(userId?.let { AppticsUser.setUser(it) }.toString())
    }

    private fun checkAndUpdateAlert(result: Result) {
        CoroutineScope(Dispatchers.IO).launch {
            val data =
                AppticsInAppUpdates.getUpdateDataIfAllConditionsSatisfied(currentActivity as FlutterActivity)
            result.success(data)
        }
    }

    private fun onClickUpdate(result: Result) {
        AppticsInAppUpdates.onClickUpdate(currentActivity as FlutterActivity)
        result.success(true)
    }

    private fun onClickReminder(result: Result) {
        AppticsInAppUpdates.onClickReminder()
        result.success(true)
    }

    private fun onClickIgnore(result: Result) {
        AppticsInAppUpdates.onClickIgnore()
        result.success(true)
    }

    private fun onClickNonSupportAlert(result: Result) {
        AppticsInAppUpdates.onClickNonSupportAlert()
        result.success(true)
    }

    private fun onSendImpressionStatus(call: MethodCall, result: Result) {
        val arg = call.arguments as Map<String, String>
        val updateId = arg["updateId"]
        if (updateId != null) {
            AppticsInAppUpdates.onSendImpressionStatus(updateId)
        }
        result.success(true)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        AppticsInAppUpdates.doOnActivityResult(resultCode, resultCode)
        return false
    }

    private fun Context.getPre(fileName: String = FILE_NAME): SharedPreferences? {
        return getSharedPreferences(fileName, Context.MODE_PRIVATE)
    }

}
