//
//  JAStoreReviewManager.m
//  JAnalytics
//
//  Created by Saravanan S on 17/01/18.
//  Copyright Â© 2018 zoho. All rights reserved.
//
#import <Foundation/Foundation.h>
#if !TARGET_OS_OSX
#import <UIKit/UIKit.h>
#else
#import <Cocoa/Cocoa.h>
#endif

#if (TARGET_OS_IOS || TARGET_OS_TV)

@interface StoreReviewWindow : UIWindow

@property (nonatomic, strong) UIView *views;

@end

@implementation StoreReviewWindow

- (instancetype)init {
    self = [super initWithFrame:[UIScreen mainScreen].bounds];
    if (self) {
        self.backgroundColor = nil;
        self.windowLevel = UIWindowLevelNormal;
    }
    return self;
}

- (BOOL)pointInside:(CGPoint)point withEvent:(UIEvent *)event {
    if (self.views == nil) {
        return NO;
    }
    
    CGPoint viewPoint = [self.views convertPoint:point fromView:self];
    return [self.views pointInside:viewPoint withEvent:event];
}

@end
#endif


#import "APStoreReviewManager.h"
#import "ARCustomHandler.h"
#import "SRModal.h"

#if __has_include(<Apptics/Apptics.h>)
#import <Apptics/ZASystemInfo.h>
#import <Apptics/Analytics.h>
#import <Apptics/ZAPIManager.h>
#import <Apptics/ZAKeyChainWrapper.h>
#import <Apptics/ZAMacros.h>
#import <Apptics/APEventsEnum.h>
#import <Apptics/ZAPrivate.h>
#import <Apptics/NSUserDefaults+SaveCustomObject.h>
#import <Apptics/APRateusObject.h>
#import <Apptics/APBundle.h>

#if (TARGET_OS_IOS || TARGET_OS_TV)
#import <Apptics/ZAnalyticsUIApplication.h>
#endif
#endif

#ifndef HAS_FEEDBACKKIT
#define HAS_FEEDBACKKIT __has_include(<AppticsFeedbackKit/FeedbackKit.h>)
#endif

UIWindow *sceneAlertWindow = nil;

@implementation APStoreReviewManager


NSString * const JAAppIncrementer         = @"JAAppIncrementer";
NSString * const JALastRemindedDate         = @"JALastRemindedDate";

NSString * const JAAppRatingShown         = @"JAAppRatingShown";
NSString * const JAFirstRatingShownDate         = @"JAFirstRatingShownDate";
NSString * const JARatedVersion         = @"JARatedVersion";
NSString * const JAExpoRemindPeriod         = @"JAExpoRemindPeriod";

bool promptForNewVersion = true;
bool inProgress = false;

NSString * const Yes         = @"Yes";
NSString * const No         = @"No";

NSString * const rateUs         = @"Rate in App Store";
NSString * const sendFeedback   = @"Send feedback";
NSString * const notNow         = @"Cancel";

NSString * bundleName = @"";
NSString * title1         = @"Rate our app";
NSString * message1 = @"Do you love the %@ app?";

NSString * title2         = @"Thank you";
NSString * message2 = @"Since you are enjoying %@, can you help us by leaving a review in the AppStore? We would love you forever, we promise.";

JRateUsSource gSource;
#define SECONDS_IN_A_DAY 86400.0

int requiredLaunchesBeforeRating         = 1;


SRModel *modals;
NSMutableDictionary *ruleBookDictionary;
NSMutableArray *ruleBookDictionaryArray;
NSMutableArray *anchors;
NSMutableArray *triggers;
bool isValidToShow;

NSMutableDictionary *engagementsAggrigateData;

NSInteger secondPromptDays;
NSInteger thirdPromptDays;
NSInteger impressionLimit;

+(APStoreReviewManager*) getInstance
{
    static APStoreReviewManager *getInstance = nil;
    
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        getInstance = [[APStoreReviewManager alloc]init];
        getInstance.enableRateUsAndReview = false;
        getInstance.shouldPauseRateUsAndReview = false;
        getInstance.showStoreAlertOnFulFillingCriteria=true;
        
        gSource = JRateUsSourceAuto;
        getInstance.zaSerialQueue = dispatch_queue_create("com.zoho.analytic.storereview.rw.serialqueue", DISPATCH_QUEUE_SERIAL);
        isValidToShow=false;
        engagementsAggrigateData = [ZAKeyChainWrapper dictionaryForKey:@"engagementsAggrigateData"].mutableCopy;
        if (engagementsAggrigateData == nil)
            engagementsAggrigateData = [NSMutableDictionary new];
        
#if !TARGET_OS_WATCH
        [[NSNotificationCenter defaultCenter] addObserver:getInstance selector:@selector(applicationDidEnterBackground:) name:UIApplicationDidEnterBackgroundNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:getInstance selector:@selector(appWillTerminate:) name:UIApplicationWillTerminateNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:getInstance selector:@selector(rateusNotification:) name:@"APRateUsNotification" object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:getInstance selector:@selector(crashFoundNotification:) name:@"APCrashFoundNotification" object:nil];
#endif
        
    });
    
    return getInstance;
}

-(void) saveAggrigatedEngagementsData{
    if (engagementsAggrigateData)
        [ZAKeyChainWrapper setDictionary:engagementsAggrigateData forKey:@"engagementsAggrigateData"];
}

- (void) enableRateUsAndReview:(BOOL)status;
{
    self.enableRateUsAndReview = status;
    [self loadRuleBook];
}

-(BOOL) shouldPromptForRating{
    return true;
}

-(void) showCustomReviewPrompt;{
    if ([self  shouldPromptForRating]) {
        bundleName = [[[APBundle getMainBundle] infoDictionary] objectForKey:@"CFBundleName"];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleNotification:) name:@"com.apticssdk.RateUsWindowClose" object:nil];
        IVLog(@"Log recieved for window closure after success");
        
        if (@available(iOS 8.0, *)) {
            [self staticReviewPopup];
        }else{
#if TARGET_OS_TV
            [self staticReviewPopup];
#endif
        }
    }
}

//-(void) staticStoreReviewPrompt;{
//
//    if ([self  shouldPromptForRating]) {
//        bundleName = [[[APBundle getMainBundle] infoDictionary] objectForKey:@"CFBundleName"];
//        if (@available(iOS 8.0, *)) {
//            [self staticReviewPopup];
//        }else{
//#if TARGET_OS_TV
//            [self staticReviewPopup];
//#endif
//        }
//    }
//}

-(void) incrementAppRuns{
    if (![self shouldUpdateData]){
        return;
    }
    int runCount = [self  getRunCounts] + 1;
    [NSUserDefaults saveObject:[NSNumber numberWithInt:runCount] key:JAAppIncrementer];
    
}

-(int) getRunCounts{
    NSNumber *savedRunCount = [NSUserDefaults loadObjectWithKey:JAAppIncrementer];
    int runCount = 0;
    if (savedRunCount != nil){
        runCount = [savedRunCount intValue];
    }
    return runCount;
}

-(void) resetAppRuns{
    [self updateExpoRemindPeriod];
    [NSUserDefaults saveObject:@0 key:JAAppIncrementer];
}

-(void) showAppStoreReviewPrompt{
    gSource = JRateUsSourceAuto;
#if (TARGET_OS_IOS || TARGET_OS_TV || TARGET_OS_WATCH)
    if (@available(iOS 10.3, *)) {
        
#if (TARGET_OS_IOS)
        dispatch_async(dispatch_get_main_queue(), ^{
            if (@available(iOS 14.0, *)) {
                for (UIWindowScene* windowScene in [ZAnalyticsUIApplication sharedApplication].connectedScenes)
                {
                    if (windowScene.activationState == UISceneActivationStateForegroundActive || windowScene.activationState == UISceneActivationStateForegroundInactive)
                    {
                        [SKStoreReviewController requestReviewInScene:windowScene];
                    }
                }
            } else {
                [SKStoreReviewController requestReview];
            }
            
            [self setAppRatingShown];
        });
#endif
        
    }else{
        //SKStoreReviewController not available so opening custom review prompt.
        [self customReviewPopup];
    }
#endif
}

-(void) takeToFeedbackScreen{
    [self updateActionForFeedback];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"APFeedbackKitShowFeedbackNotification" object:nil];
#if !HAS_FEEDBACKKIT
    IVLogWarn(@"FeedbackKit framework is missing, please fix and try");
#endif
    
}


//MARK: notification reciever for RateUs Window Close

- (void)handleNotification:(NSNotification *)notification {
    dispatch_async(dispatch_get_main_queue(), ^{
        sceneAlertWindow.hidden = YES;
    });
}






#pragma mark - App Store Control
-(void) getAppStoreReviewUrl:(void (^)(NSURL * _Nullable storeReviewUrl))completionHandler{
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"lookup?bundleId=%@",[ZASystemInfo getBundleID]] relativeToURL:[NSURL URLWithString:@"https://itunes.apple.com"]]];
    [request setHTTPMethod:@"GET"];
    [request setHTTPShouldHandleCookies:NO];
    
    NSURLSession *session = [NSURLSession sessionWithConfiguration:[NSURLSessionConfiguration defaultSessionConfiguration] delegate:nil delegateQueue:nil];
    
    NSURLSessionDataTask *task = [session dataTaskWithRequest:request completionHandler:^(NSData * data, NSURLResponse * response, NSError * error) {
        
        if(error == nil)
        {
            NSError *jsonParsingError;
            NSDictionary *JSONData = [NSJSONSerialization JSONObjectWithData:data options:0 error:&jsonParsingError];
            
            if(jsonParsingError == nil)
            {
                NSHTTPURLResponse *httpResponse = (NSHTTPURLResponse*)response;
                if ([httpResponse statusCode] == 200)
                {
                    if ([[JSONData objectForKey:@"results"] isKindOfClass:[NSArray class]]){
                        NSArray *results = (NSArray*) [JSONData objectForKey:@"results"];
                        if ([results count] > 0){
                            NSDictionary *appInfo =  results.firstObject;
                            NSString *trackViewUrl = (NSString *)appInfo[@"trackViewUrl"];
                            NSString *actionParam = @"action=write-review";
                            if (trackViewUrl != nil) {
                                if ([[trackViewUrl componentsSeparatedByString:@"?"] count] > 0){
                                    trackViewUrl = [NSString stringWithFormat:@"%@&%@",trackViewUrl,actionParam];
                                }else{
                                    trackViewUrl = [NSString stringWithFormat:@"%@?%@",trackViewUrl,actionParam];
                                }
                                NSURL *iTunesURL = [NSURL URLWithString:trackViewUrl];
                                completionHandler(iTunesURL);
                            }
                        }
                    }
                }
            }
        }
        
    }];
    [task resume];
}

-(void) takeToAppStoreReviewScreen{
//    [self updateActionForAppStoreReview];
    [self getAppStoreReviewUrl:^(NSURL * _Nullable storeReviewUrl) {
        dispatch_async(dispatch_get_main_queue(), ^{
            if (storeReviewUrl != nil){
#if (TARGET_OS_IOS || TARGET_OS_TV || TARGET_OS_WATCH)
                
                [[ZAnalyticsUIApplication getInstance] za_openURL:storeReviewUrl options:nil completionHandler:^(BOOL success) {
                    
                }];
#endif
            }
        });
    }];
}

-(void) updateActionForAppStoreReview{
    [self updateRateUAction:(JRateUsActionRateOnStore) type:(JRateUsTypePopup1) source:gSource];
}

-(void) updateActionForFeedback{
    [self updateRateUAction:(JRateUsActionSendFeedback) type:(JRateUsTypePopup1) source:gSource];
}

#pragma mark - App Rating Shown

-(void) firstRatingShownDate{
    if ([self  getFirstRatingShownDate] == nil){
        [NSUserDefaults saveObject:[NSDate date] key:JAFirstRatingShownDate];
    }
}

-(NSDate*) getFirstRatingShownDate{
    return [NSUserDefaults loadObjectWithKey:JAFirstRatingShownDate];
}

-(void) lastRemindedDate{
    [self  firstRatingShownDate];
    [NSUserDefaults saveObject:[NSDate date] key:JALastRemindedDate];
}

-(NSDate*) getLastRemindedDate{
    return [NSUserDefaults loadObjectWithKey:JALastRemindedDate];
}

-(void) setAppRatingShown{
    
    [ZAKeyChainWrapper setNumber:self.gNumberOfAttempts forKey:kJA_autoReviewAttempt];
    [self updateRateUAction:(JRateUsActionRateOnStore) type:(JRateUsTypePopup1) source:gSource];
    [self resetAllNumbers];
    [self setRatedVersion:[ZASystemInfo getAppVersion]];
    [NSUserDefaults saveObject:@true key:JAAppRatingShown];
}

-(BOOL) hasAppRatingShown{
    return ([NSUserDefaults loadObjectWithKey:JAAppRatingShown] != nil);
}

-(void) setRatedVersion : (NSString*) version{
    [self firstRatingShownDate];
    [NSUserDefaults saveObject:version key:JARatedVersion];
}

-(NSString*) getRatedVersion{
    return [NSUserDefaults loadObjectWithKey:JARatedVersion];
}

-(BOOL) ratedThisVersion{
    return [[self  getRatedVersion] isEqual:[ZASystemInfo getAppVersion]];
}

-(BOOL) ratedAnyVersion{
    return ([self  getRatedVersion] != nil);
}

-(BOOL) remindedToday{
    NSDate *lastRemindedDate = [self  getLastRemindedDate];
    return (lastRemindedDate != nil && ([[NSDate date] timeIntervalSinceDate:lastRemindedDate] < [self  getExpoRemindPeriod] * SECONDS_IN_A_DAY));
}

-(void) updateExpoRemindPeriod{
    int expoValue = [self  getExpoRemindPeriod] + 2;
    [NSUserDefaults saveObject:[NSNumber numberWithInt:expoValue] key:JAExpoRemindPeriod];
}

-(int) getExpoRemindPeriod{
    NSNumber *value = [NSUserDefaults loadObjectWithKey:JAExpoRemindPeriod];
    return (value != nil) ? value.intValue : 1;
}
#if (TARGET_OS_IOS)

-(void) elseWindowCall{
    UIWindow *alertWindow = [[UIWindow alloc] initWithFrame:[UIScreen mainScreen].bounds];
    alertWindow.rootViewController = [[UIViewController alloc] init];
    UIAlertController* alertController = [self  createAlertView:nil description:ZAnalyticsLocalizedString(@"zanalytics.rateus.prompt.title")];
    [alertController addAction:[self  addGoToItunesAction:[ZASystemInfo getBundleID] andTitle:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.rateinappstore")]];
    [alertController addAction:[self  addSendFeedbackAction:[ZASystemInfo getBundleID] andTitle:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.openfeedback")]];
    [alertController addAction:[UIAlertAction actionWithTitle:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.later") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        dispatch_async(dispatch_get_main_queue(), ^{
             alertWindow.hidden = YES;
             [alertWindow removeFromSuperview];

        });
    }]];
    alertWindow.windowLevel = UIWindowLevelAlert + 1;
    [alertWindow makeKeyAndVisible];
    [alertWindow.rootViewController presentViewController:alertController animated:YES completion:nil];
}
#endif


#if (TARGET_OS_IOS)

-(void) methodWindowCreate{
    if (@available(iOS 13.0, *)) {
            StoreReviewWindow *floatingBottomWindow = [[StoreReviewWindow alloc] init];
            sceneAlertWindow = floatingBottomWindow;        
        UIWindowScene *currentWindowScene = (UIWindowScene *)[UIApplication.sharedApplication.connectedScenes.allObjects firstObject];
        if (currentWindowScene) {
            sceneAlertWindow = [[UIWindow alloc] initWithWindowScene:currentWindowScene];
            sceneAlertWindow.windowLevel = UIWindowLevelAlert + 1;
            sceneAlertWindow.rootViewController = [[UIViewController alloc] init];
            [sceneAlertWindow makeKeyAndVisible];
            UIAlertController* alertController = [self  createAlertView:nil description:ZAnalyticsLocalizedString(@"zanalytics.rateus.prompt.title")];
            [alertController addAction:[self  addGoToItunesAction:[ZASystemInfo getBundleID] andTitle:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.rateinappstore")]];
            [alertController addAction:[self  addSendFeedbackAction:[ZASystemInfo getBundleID] andTitle:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.openfeedback")]];
            [alertController addAction:[UIAlertAction actionWithTitle:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.later") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    sceneAlertWindow.hidden = YES;
                });
            }]];
            [sceneAlertWindow.rootViewController presentViewController:alertController animated:YES completion:nil];
        }
        else{
            [self elseWindowCall];
        }
        
    }
    else{
        [self elseWindowCall];
    }
}

#endif



#pragma mark - Rate The App

-(void) customReviewPopup{
    gSource = JRateUsSourceAuto;
    #if (TARGET_OS_IOS)
//        UIAlertController* alert = [self  createAlertView:nil description:ZAnalyticsLocalizedString(@"zanalytics.rateus.prompt.title")];
//        [alert addAction:[self addGoToItunesAction:[ZASystemInfo getBundleID] andTitle:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.rateinappstore")]];
//        [alert addAction:[self  addSendFeedbackAction:[ZASystemInfo getBundleID] andTitle:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.openfeedback")]];
//        [alert addAction:[self  addSkipAction:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.later")]];
//
//        dispatch_async(dispatch_get_main_queue(), ^{
//            UIViewController *presentingViewController = [ZAnalyticsUIApplication topPresentedViewController];
//            if ([ZAnalyticsUIApplication canPresentFromViewController:presentingViewController]) {
//                [self setAppRatingShown];
//                [presentingViewController presentViewController:alert animated:YES completion:nil];
//            }
//        });
    [self methodWindowCreate];
    [self setAppRatingShown];
    
    #endif
    
   
}

-(void) staticReviewPopup{
  gSource = JRateUsSourceManual;
#if (TARGET_OS_IOS)
//    UIAlertController* alert = [self createAlertView:nil description:ZAnalyticsLocalizedString(@"zanalytics.rateus.prompt.title")];
//    [alert addAction:[self addGoToItunesAction:[ZASystemInfo getBundleID] andTitle:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.rateinappstore")]];
//    [alert addAction:[self addSendFeedbackAction:[ZASystemInfo getBundleID] andTitle:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.openfeedback")]];
//    [alert addAction:[self  addSkipAction:ZAnalyticsLocalizedString(@"zanalytics.rateus.action.later")]];
//
//    dispatch_async(dispatch_get_main_queue(), ^{
//        UIViewController *presentingViewController = [ZAnalyticsUIApplication topPresentedViewController];
//
//        if ([ZAnalyticsUIApplication canPresentFromViewController:presentingViewController]) {
//            [presentingViewController presentViewController:alert animated:YES completion:nil];
//        }
//    });
    [self methodWindowCreate];
    
#endif
}

-(void) presentSecondPopup{
#if (TARGET_OS_IOS)
    //    UIAlertController* alert = [self createAlertView:title2 description:[NSString stringWithFormat:message2, bundleName]];
    //    [alert addAction:[self  addGoToItunesAction:[ZASystemInfo getBundleID] andTitle:rateUs]];
    //    [alert addAction:[self  addSkipAction:notNow]];
    //
    //    dispatch_async(dispatch_get_main_queue(), ^{
    //        UIViewController *presentingViewController = [ZAnalyticsUIApplication topPresentedViewController];
    //
    //        if ([ZAnalyticsUIApplication canPresentFromViewController:presentingViewController]) {
    //            [presentingViewController presentViewController:alert animated:YES completion:nil];
    //        }//else skip this time
    //    });
#endif
}

#pragma mark - UIAlertView

#if (TARGET_OS_IOS)
-(UIAlertController*) createAlertView : (NSString*) title description : (NSString*) description{
    return [UIAlertController alertControllerWithTitle:title
                                               message:description
                                        preferredStyle:UIAlertControllerStyleAlert];
}

#pragma mark - UIAlertActions1

-(UIAlertAction*) addLikeAction : (NSString*) bundleId andTitle : (NSString*) title  {
    return [UIAlertAction actionWithTitle:title style:UIAlertActionStyleDefault
                                  handler:^(UIAlertAction * action) {
        // call the second popup
        [self  presentSecondPopup];
    }];
}

-(UIAlertAction*) addUnLikeAction : (NSString*) title {
    return [UIAlertAction actionWithTitle:title style:UIAlertActionStyleCancel
                                  handler:^(UIAlertAction * action) {
        // take him to the feedback
        
    }];
}

#pragma mark - UIAlertActions2

-(UIAlertAction*) addGoToItunesAction : (NSString*) bundleId andTitle : (NSString*) title  {
    return [UIAlertAction actionWithTitle:title style:UIAlertActionStyleDefault
                                  handler:^(UIAlertAction * action) {
        //dismiss window
        [[NSNotificationCenter defaultCenter]postNotificationName:@"com.apticssdk.RateUsWindowClose" object:nil];

        [ARCustomHandlerManager rateUsActionCompletionHandler:JRateUsActionRateOnStore];
    }];
}

-(UIAlertAction*) addSendFeedbackAction : (NSString*) bundleId andTitle : (NSString*) title  {
    return [UIAlertAction actionWithTitle:title style:UIAlertActionStyleDefault
                                  handler:^(UIAlertAction * action) {
        [ARCustomHandlerManager rateUsActionCompletionHandler:JRateUsActionSendFeedback];
        
    }];
}

-(UIAlertAction*) addSkipAction : (NSString*) title {
    return [UIAlertAction actionWithTitle:title style:UIAlertActionStyleCancel
                                  handler:^(UIAlertAction * action) {
        [self  updateRateUAction:(JRateUsActionCancel) type:(JRateUsTypePopup1) source:gSource];
    }];
}

#endif

#pragma mark - NetworkCalls

-(void) updateRateUAction: (JRateUsAction) action type : (JRateUsType) type source : (JRateUsSource) source{
    
    NSNumber *numberOfAttempts = @(-1);
//    if (source == JRateUsSourceAuto){
        numberOfAttempts = [ZAKeyChainWrapper numberForKey:kJA_autoReviewAttempt];
        if (numberOfAttempts == nil){
            numberOfAttempts = @1;
        }
//        action = -1;
//    }
    if (self.criteriaId != nil)
        [APRateusObject addObjectToQueueForDictionary:@{@"criteriaid":self.criteriaId, @"popupsource":@(source), @"popupaction": @(action), @"popupnumber": numberOfAttempts}];
    else
        [APRateusObject addObjectToQueueForDictionary:@{@"criteriaid":@0, @"popupsource":@(source), @"popupaction": @(action), @"popupnumber": numberOfAttempts}];
    
}

- (NSString *)extractMinNumberFromText:(NSString *)text
{
    NSError *error = nil;
    NSRegularExpression *regex = [NSRegularExpression regularExpressionWithPattern:@".+?(?=-)" options:NSRegularExpressionCaseInsensitive error:&error];
    
    NSRange textRange = NSMakeRange(0, text.length);
    NSRange matchRange = [regex rangeOfFirstMatchInString:text options:NSMatchingReportProgress range:textRange];
    
    // Did we find a matching range
    if (matchRange.location != NSNotFound){
        return [text substringWithRange:matchRange];
    }else{
        return nil;
    }
}

-(void) incrementSessionHitCount : (NSInteger) sessionDuration{
    //Check for duration here
    if (![self shouldUpdateData]){
        return;
    }
    for (ruleBookDictionary in ruleBookDictionaryArray) {
        SRSessionduration* duration = [ruleBookDictionary objectForKey:[NSString stringWithFormat:@"%@_duration",kJA_sessionHit]];
        if (duration != nil){
            NSInteger minNumber = duration.min;
            if (sessionDuration > minNumber){
                NSNumber *count = [self numberForKey:kJA_sessionHit];
                if (count == nil)
                    count = @(0);
                
                count = @([count intValue] + 1);
                [self setNumber:count forKey:kJA_sessionHit];
            }
        }
    }
}

-(void) incrementEventHitCountForEventId : (NSString*) eventId{
    
    if (eventId){
        
        if ([APBundle getInstance].eventsProtocolClass && [[APBundle getInstance].eventsProtocolClass respondsToSelector:@selector(formatTypeToPrivateObject:event:)]){
            APPrivateObject *eventObject = [[APBundle getInstance].eventsProtocolClass formatTypeToPrivateObjectFromEventId: eventId];
            
            if (eventObject){
                NSString *event = eventObject.event;
                NSString *group = eventObject.group;
                if (group && event){
                    [self incrementEventHitCount:[NSString stringWithFormat:@"%@_%@",group.lowercaseString,event.lowercaseString]];
                }
            }
            
        }
        
    }
}

-(void) incrementEventHitCount : (NSString*) name{
    if (![self shouldUpdateData]){
        return;
    }
    NSString *key = [NSString stringWithFormat:@"%@_%@",kJA_eventHit, name];
    NSNumber *count = [self numberForKey:key];
    if (count == nil)
        count = @(0);
    
    count = @([count intValue] + 1);
    [self setNumber:count forKey:key];
    [self checkIfCriteriaMatches:key];
}

-(void) incrementScreenHitCount : (NSString*) name;
{
    if (![self shouldUpdateData]){
        return;
    }
    
    NSString *key = [NSString stringWithFormat:@"%@_%@",kJA_screenHit, name.lowercaseString];
    NSNumber *count = [self numberForKey:key];
    if (count == nil)
        count = @(0);
    
    count = @([count intValue] + 1);
    [self setNumber:count forKey:key];
    
    [self checkIfCriteriaMatches:key];
}

-(void) checkIfCriteriaMatches : (NSString*) key{
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 2 * NSEC_PER_SEC), self.zaSerialQueue, ^{
        if (!inProgress && self.showStoreAlertOnFulFillingCriteria){
            inProgress = true;
            [self validateAndShowReviewPrompt:key];
        }
    });
}

-(void) validateAndShowReviewPrompt : (NSString*) key{
    
    if (self.shouldPauseRateUsAndReview || !self.enableRateUsAndReview || ruleBookDictionaryArray == nil){
        IVLog(@"returning false on validateAndShowReviewPrompt %@",key);
        inProgress = false;
        dispatch_async(dispatch_get_main_queue(), ^{
            [ARCustomHandlerManager cancelledReviewPrompt:^{}];
        });
        return;
    }
    
    if ((triggers != nil && [triggers containsObject:key]) || (anchors != nil && [anchors containsObject:key]) || key == nil){
        
        dispatch_async(self.zaSerialQueue, ^{
            
//            bool isValidToShow = false;
            NSNumber *criteriaIdFromKeychain;
            
            for (NSMutableDictionary *ruleBookDict in ruleBookDictionaryArray) {
                
                criteriaIdFromKeychain = ruleBookDict[@"criteriaid"];
                self.criteriaId = criteriaIdFromKeychain;
                NSString *type = ruleBookDict[@"type"];
                NSNumber *targetGoalScore = ruleBookDict[@"goalscore"];
                
                NSArray *anchorpoints = ruleBookDict[@"anchorpoint"];
                
                NSMutableArray *items = ruleBookDict.allKeys.mutableCopy;
                [items removeObjectsInArray:@[@"criteriaid", @"type", @"goalscore", @"anchorpoint"]];
                
                NSNumber *totalGoalScore = @(0);
                
                for (NSString* _key in items){
                    NSNumber *weightage = [ruleBookDict objectForKey:_key];
                    NSNumber *hitCount = @(0);
                    
                    if (![_key isEqualToString:@"kJA_sessionHit_duration"] && weightage != nil && weightage.intValue > 0){
                        
                        if ([_key isEqualToString:kJA_sessionHit]){
                            hitCount = [self numberForKey:kJA_sessionHit];
                        }else if ([_key hasPrefix:kJA_eventHit]){
                            hitCount = [self numberForKey:_key];
                        }else if ([_key hasPrefix:kJA_screenHit]){
                            hitCount = [self numberForKey:_key];
                        }
                        
                        if ([type isEqual:@"hitbased"]){
                            if (hitCount.integerValue >= weightage.integerValue){
                                isValidToShow = true;
                            }else{
                                isValidToShow = false;
                                break;
                            }
                        }else{
                            
                            totalGoalScore = @(totalGoalScore.floatValue + (hitCount.floatValue * weightage.floatValue));
                            
                            if (totalGoalScore.floatValue >= targetGoalScore.floatValue){
                                isValidToShow = true;
                                break;
                            }else{
                                isValidToShow = false;
                            }
                        }
                    }
                }
                if (isValidToShow){
                    if (anchorpoints.count > 0 && key != nil){
                        if ([anchorpoints containsObject: key]){
                            break;
                        }else{
                            isValidToShow = false;
                        }
                    }else{
                        break;
                    }
                }
                
            }
            
            if (!isValidToShow)
            {
                IVLog(@"returning false on validateAndShowReviewPrompt %i", isValidToShow);
                inProgress = false;
                dispatch_async(dispatch_get_main_queue(), ^{
                    [ARCustomHandlerManager cancelledReviewPrompt:^{}];
                });
                return;
            }
            
            NSNumber *numberOfAttempts = [ZAKeyChainWrapper numberForKey:kJA_autoReviewAttempt];
            NSNumber *daysCount = @0;
            NSString *promptedDateString = [ZAKeyChainWrapper stringForKey:kJA_autoPromptedDate];
            
            if (promptedDateString){
                NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
                [dateFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"]];
                [dateFormatter setDateFormat:@"dd-MM-yyyy"];
                NSDate *promptedDate = [dateFormatter dateFromString:promptedDateString];
                NSTimeInterval secondsBetween = [[NSDate date] timeIntervalSinceDate:promptedDate];
                daysCount = @((int)(secondsBetween / 86400));
            }
            
            if (numberOfAttempts == nil){
                numberOfAttempts = @0;
            }
            
            if (numberOfAttempts.integerValue == 1){
                if (daysCount.integerValue < secondPromptDays){
                    IVLog(@"returning false on validateAndShowReviewPrompt daysCount %i with secondPromptDays %i", daysCount.integerValue, secondPromptDays);
                    inProgress = false;
                    dispatch_async(dispatch_get_main_queue(), ^{
                        [ARCustomHandlerManager cancelledReviewPrompt:^{}];
                    });
                    return;
                }
            }
            
            if (numberOfAttempts.integerValue == 2){
                if (daysCount.integerValue < thirdPromptDays){
                    IVLog(@"returning false on validateAndShowReviewPrompt daysCount %i with secondPromptDays %i", daysCount.integerValue, thirdPromptDays);
                    inProgress = false;
                    dispatch_async(dispatch_get_main_queue(), ^{
                        [ARCustomHandlerManager cancelledReviewPrompt:^{}];
                    });
                    return;
                }
            }
            
            if (numberOfAttempts.integerValue == 3){
                if (daysCount.integerValue > 365){
                    [ZAKeyChainWrapper setNumber:@0 forKey:kJA_autoReviewAttempt];
                }
                IVLog(@"returning false on validateAndShowReviewPrompt daysCount %i with 365", daysCount.integerValue);
                inProgress = false;
                dispatch_async(dispatch_get_main_queue(), ^{
                    [ARCustomHandlerManager cancelledReviewPrompt:^{}];
                });
                return;
            }
            
            self.gNumberOfAttempts = [NSNumber numberWithInteger:numberOfAttempts.integerValue + 1];
            dispatch_async(dispatch_get_main_queue(), ^{
                [ARCustomHandlerManager willDisplayReviewPrompt:^{
                    [self showAppStoreReviewPrompt];
                }];
                inProgress = false;
            });
            
        });
    }else{
        inProgress = false;
        dispatch_async(dispatch_get_main_queue(), ^{
            [ARCustomHandlerManager cancelledReviewPrompt:^{}];
        });
    }        
}

/*
 -(void) validateAndShowReviewPrompt{
 
 //  if (!self.enableRateUsAndReview || [self ratedThisVersion] || ruleBookDictionaryArray == nil){
 if (!self.enableRateUsAndReview || ruleBookDictionaryArray == nil){
 inProgress = false;
 return;
 }
 
 dispatch_async(self.zaSerialQueue, ^{
 
 bool isValidToShow = false;
 NSNumber *criteriaIdFromKeychain;
 
 for (NSMutableDictionary *ruleBookDict in ruleBookDictionaryArray) {
 for (NSString* _key in ruleBookDict.allKeys){
 NSNumber *limit = [ruleBookDict objectForKey:_key];
 NSNumber *hitCount = @(0);
 
 if ([_key containsString:@"criteriaid"]) {
 criteriaIdFromKeychain = ruleBookDict[_key];
 criteriaId = criteriaIdFromKeychain;
 }else{
 if (![_key isEqualToString:@"kJA_sessionHit_duration"] && limit != nil && limit.intValue > 0){
 
 if ([_key isEqualToString:kJA_sessionHit]){
 hitCount = [self numberForKey:kJA_sessionHit];
 }else if ([_key hasPrefix:kJA_eventHit]){
 hitCount = [self numberForKey:_key];
 }else if ([_key hasPrefix:kJA_screenHit]){
 hitCount = [self numberForKey:_key];
 }
 
 if (hitCount.integerValue >= limit.integerValue){
 isValidToShow = true;
 }else{
 isValidToShow = false;
 break;
 }
 }
 }
 }
 
 if (isValidToShow){
 break;
 }
 }
 
 if (!isValidToShow)
 {
 IVLog(@"returning false on validateAndShowReviewPrompt %i", isValidToShow);
 inProgress = false;
 criteriaId = @0;
 return;
 }
 
 NSNumber *numberOfAttempts = [ZAKeyChainWrapper numberForKey:kJA_autoReviewAttempt];
 NSNumber *daysCount = @0;
 NSString *promptedDateString = [ZAKeyChainWrapper stringForKey:kJA_autoPromptedDate];
 
 if (promptedDateString){
 NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
 [dateFormatter setDateFormat:@"dd-MM-yyyy"];
 NSDate *promptedDate = [dateFormatter dateFromString:promptedDateString];
 NSTimeInterval secondsBetween = [[NSDate date] timeIntervalSinceDate:promptedDate];
 daysCount = @((int)(secondsBetween / 86400));
 }
 
 if (numberOfAttempts == nil){
 numberOfAttempts = @0;
 }
 
 if (numberOfAttempts.integerValue == 1){
 if (daysCount.integerValue < secondPromptDays){
 IVLog(@"returning false on validateAndShowReviewPrompt daysCount %i with secondPromptDays %i", daysCount.integerValue, secondPromptDays);
 inProgress = false;
 criteriaId = @0;
 return;
 }
 }
 
 if (numberOfAttempts.integerValue == 2){
 if (daysCount.integerValue < thirdPromptDays){
 IVLog(@"returning false on validateAndShowReviewPrompt daysCount %i with secondPromptDays %i", daysCount.integerValue, thirdPromptDays);
 inProgress = false;
 criteriaId = @0;
 return;
 }
 }
 
 if (numberOfAttempts.integerValue == 3){
 if (daysCount.integerValue > 365){
 [ZAKeyChainWrapper setNumber:@0 forKey:kJA_autoReviewAttempt];
 }
 IVLog(@"returning false on validateAndShowReviewPrompt daysCount %i with 365", daysCount.integerValue);
 inProgress = false;
 criteriaId = @0;
 return;
 }
 
 gNumberOfAttempts = [NSNumber numberWithInteger:numberOfAttempts.integerValue + 1];
 dispatch_async(dispatch_get_main_queue(), ^{
 [ARCustomHandlerManager willDisplayReviewPrompt:^{
 [self showAppStoreReviewPrompt];
 //Resetting the criteria value;
 criteriaId = @0;
 }];
 inProgress = false;
 });
 
 });
 
 }
 */

-(void) storeReviewMasterReset{
    [ZAKeyChainWrapper removeItemForKey:kJA_autoPromptedDate];
    [ZAKeyChainWrapper setNumber:@0 forKey:kJA_autoReviewAttempt];
    [NSUserDefaults saveObject:@"0.0" key:JARatedVersion];
    //  for (NSMutableDictionary *ruleBookDict in ruleBookDictionaryArray) {
    for (NSString* _key in engagementsAggrigateData.allKeys){
        NSNumber *hitCount = @(0);
        if ([_key isEqualToString:kJA_sessionHit]){
            [self setNumber:hitCount forKey:kJA_sessionHit];
        }else if ([_key hasPrefix:kJA_eventHit]){
            [self setNumber:hitCount forKey:_key];
        }else if ([_key hasPrefix:kJA_screenHit]){
            [self setNumber:hitCount forKey:_key];
        }
    }
    //  }
}

-(void) resetAllNumbers{
    
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"dd-MM-yyyy"];
    [dateFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"]];
    NSString *stringDate = [dateFormatter stringFromDate:[NSDate date]];
    [ZAKeyChainWrapper setString:stringDate forKey:kJA_autoPromptedDate];
    //    for (NSMutableDictionary *ruleBookDict in ruleBookDictionaryArray) {
    for (NSString* _key in engagementsAggrigateData.allKeys){
        NSNumber *hitCount = @(0);
        if ([_key isEqualToString:kJA_sessionHit]){
            [self setNumber:hitCount forKey:kJA_sessionHit];
        }else if ([_key hasPrefix:kJA_eventHit]){
            [self setNumber:hitCount forKey:_key];
        }else if ([_key hasPrefix:kJA_screenHit]){
            [self setNumber:hitCount forKey:_key];
        }
    }
    //    }
}

/*
 -(void) loadRuleBookold{
 
 dispatch_async(self.zaSerialQueue, ^{
 
 if (!self.enableRateUsAndReview){
 return;
 }
 @try {
 NSNumber *numberOfAttempts = [ZAKeyChainWrapper numberForKey:kJA_autoReviewAttempt];
 if (numberOfAttempts == nil){
 [ZAKeyChainWrapper setNumber:@0 forKey:kJA_autoReviewAttempt];
 }
 
 NSDictionary *info = [ZAKeyChainWrapper dictionaryForKey:@"rateUsData"];
 
 if (info == nil || info.count == 0)
 return;
 
 NSDictionary *dataDict = info;//[info objectForKey:@"data"];
 NSArray *alertCriteria = [dataDict objectForKey:@"criteria"];
 impressionLimit = za_safeNumber((NSNumber*)[dataDict objectForKey:@"impressions"]).integerValue;
 secondPromptDays = za_safeNumber((NSNumber*)[dataDict objectForKey:@"SecondPromptDays"]).integerValue;
 thirdPromptDays = za_safeNumber((NSNumber*)[dataDict objectForKey:@"ThirdPromptDays"]).integerValue;
 
 NSError *error;
 NSMutableData *jsonData = [[NSJSONSerialization dataWithJSONObject:@{@"criteria":alertCriteria} options:0 error:&error] copy];
 
 modals = [SRModel fromData:jsonData error:&error];
 
 
 if (error == nil){
 
 if (modals.criteria.count == 0)
 return;
 ruleBookDictionaryArray = [[NSMutableArray alloc] init];
 triggers = [[NSMutableArray alloc] init];
 [ZAKeyChainWrapper setNumber:za_safeLong(@(impressionLimit)) forKey:kJA_autoReviewImpressionLimit];
 
 for (SRCriterion *alertCriterion in modals.criteria) {
 ruleBookDictionary = [[NSMutableDictionary alloc] init];
 [ruleBookDictionary setObject:[NSNumber numberWithInteger:alertCriterion.criteriaid] forKey:@"criteriaid"];
 SRSession *session = alertCriterion.session;
 if ([self isHitCountValid:session.hitcount]) {
 [ruleBookDictionary setObject:[NSNumber numberWithInteger:session.hitcount] forKey:kJA_sessionHit];
 [ruleBookDictionary setObject:session.sessionduration forKey:[NSString stringWithFormat:@"%@_duration",kJA_sessionHit]];
 [self setNumberIfNotExist:@0 forKey:kJA_sessionHit];
 
 }
 
 for (SREventconf *eventsConf in alertCriterion.eventconf){
 for (SREvent *event in eventsConf.events){
 NSString *key;
 if (eventsConf.group){
 key = [NSString stringWithFormat:@"%@_%@_%@",kJA_eventHit,eventsConf.group.lowercaseString,event.name.lowercaseString];
 }else{
 key = [NSString stringWithFormat:@"%@_j_default_%@",kJA_eventHit,event.name.lowercaseString];
 }
 if ([self isHitCountValid:eventsConf.hitcount]) {
 [ruleBookDictionary setObject:[NSNumber numberWithInteger:eventsConf.hitcount] forKey:key];
 [triggers addObject:key];
 [self setNumberIfNotExist:@0 forKey:key];
 }
 }
 }
 
 for (SRScreenconf *screen in alertCriterion.screenconf){
 for (SRScreen *screenname in screen.screens){
 NSString *key;
 key = [NSString stringWithFormat:@"%@_%@",kJA_screenHit,screenname.name.lowercaseString];
 if ([self isHitCountValid:screen.hitcount]) {
 [ruleBookDictionary setObject:[NSNumber numberWithInteger:screen.hitcount] forKey:key];
 [triggers addObject:key];
 [self setNumberIfNotExist:@0 forKey:key];
 }
 }
 }
 
 [ruleBookDictionaryArray addObject:ruleBookDictionary];
 }
 }
 } @catch (NSException *exception) {
 IVLog(@"Exception occured while parsing rateus data - %@",exception);
 }
 });
 }
 */

-(void) loadRuleBook{
    
    dispatch_async(self.zaSerialQueue, ^{
        
        if (!self.enableRateUsAndReview){
            return;
        }
        @try{
            NSNumber *numberOfAttempts = [ZAKeyChainWrapper numberForKey:kJA_autoReviewAttempt];
            if (numberOfAttempts == nil){
                [ZAKeyChainWrapper setNumber:@0 forKey:kJA_autoReviewAttempt];
            }
            
            NSDictionary *_info = [ZAKeyChainWrapper dictionaryForKey:@"rateUsData"];
            
            if (_info == nil || _info.count == 0)
                return;
            
            id info;
            
            NSData *preData = [ZAKeyChainWrapper dataForKey:@"appRatingData"];
            NSData *data = [NSKeyedArchiver archivedDataWithRootObject:_info];
            
            if (![preData isEqualToData:data]){
                info = _info;
                [ZAKeyChainWrapper setData:data forKey:@"appRatingData"];
            }else{
                NSDictionary *prevInfo = (NSDictionary*) [NSKeyedUnarchiver unarchiveObjectWithData:preData];
                if (prevInfo != nil) {
                    info = prevInfo;
                }
            }
            
            dispatch_async(self.zaSerialQueue, ^{
                
                NSDictionary *dataDict = info;//[info objectForKey:@"data"];
                NSDictionary *alertCriteria = [dataDict objectForKey:@"criteria"];
                
                impressionLimit = za_safeNumber((NSNumber*)[dataDict objectForKey:@"impressions"]).integerValue;
                secondPromptDays = za_safeNumber((NSNumber*)[dataDict objectForKey:@"SecondPromptDays"]).integerValue;
                thirdPromptDays = za_safeNumber((NSNumber*)[dataDict objectForKey:@"ThirdPromptDays"]).integerValue;
                
                //                for (NSDictionary *criteria in alertCriteria){
                
                NSArray *hitebase = alertCriteria[@"hitbased"];
                NSArray *scorebased = alertCriteria[@"scorebased"];
                
                if (hitebase.count == 0 && scorebased.count == 0)
                    return;
                
                ruleBookDictionaryArray = [[NSMutableArray alloc] init];
                anchors = [[NSMutableArray alloc] init];
                triggers = [[NSMutableArray alloc] init];
                [ZAKeyChainWrapper setNumber:za_safeLong(@(impressionLimit)) forKey:kJA_autoReviewImpressionLimit];
                for (NSDictionary *set in hitebase){
                    ruleBookDictionary = [[NSMutableDictionary alloc] init];
                    [ruleBookDictionary setObject:set[@"criteriaid"] forKey:@"criteriaid"];
                    [ruleBookDictionary setObject:@"hitbased" forKey:@"type"];
                    NSDictionary *session = set[@"session"];
                    NSNumber *hitcount = session[@"weightage"];
                    NSDictionary *sessionduration = session[@"sessionduration"];
                    if ([self isHitCountValid:hitcount.integerValue]) {
                        [ruleBookDictionary setObject:hitcount forKey:kJA_sessionHit];
                        [ruleBookDictionary setObject:sessionduration forKey:[NSString stringWithFormat:@"%@_duration",kJA_sessionHit]];
                        [self setNumberIfNotExist:@0 forKey:kJA_sessionHit];
                    }
                    
                    NSMutableArray *_anchors = [NSMutableArray new];
                    NSArray *events = [set valueForKeyPath:@"anchorpoint.eventconf.events"];
                    NSString *group = [set valueForKeyPath:@"anchorpoint.eventconf.group"];
                    for (NSDictionary *event in events){
                        NSString *name = event[@"name"];
                        NSString *key;
                        if (group){
                            key = [NSString stringWithFormat:@"%@_%@_%@",kJA_eventHit,group.lowercaseString,name.lowercaseString];
                        }else{
                            key = [NSString stringWithFormat:@"%@_%@",kJA_eventHit,name.lowercaseString];
                        }
                        
                        [_anchors addObject:key];
                    }
                    NSArray *screens = [set valueForKeyPath:@"anchorpoint.screenconf.screens"];
                    for (NSDictionary *screen in screens){
                        NSString *screenname = screen[@"name"];
                        NSString *key = [NSString stringWithFormat:@"%@_%@",kJA_screenHit,screenname.lowercaseString];
                        [_anchors addObject:key];
                        
                    }
                    [anchors addObjectsFromArray:_anchors];
                    [ruleBookDictionary setObject:_anchors forKey:@"anchorpoint"];
                    
                    for (NSDictionary *eventconf in set[@"eventconf"]){
                        NSNumber *weightage = eventconf[@"weightage"];
                        NSString *group = eventconf[@"group"];
                        for (NSDictionary *event in eventconf[@"events"]){
                            NSString *name = event[@"name"];
                            NSString *key;
                            if (group){
                                key = [NSString stringWithFormat:@"%@_%@_%@",kJA_eventHit,group.lowercaseString,name.lowercaseString];
                            }else{
                                key = [NSString stringWithFormat:@"%@_%@",kJA_eventHit,name.lowercaseString];
                            }
                            if ([self isHitCountValid:weightage.integerValue]) {
                                [ruleBookDictionary setObject:weightage forKey:key];
                                [triggers addObject:key];
                                [self setNumberIfNotExist:@0 forKey:key];
                            }
                            
                        }
                    }
                    
                    for (NSDictionary *screenconf in set[@"screenconf"]){
                        NSNumber *weightage = screenconf[@"weightage"];
                        for (NSDictionary *screen in screenconf[@"screens"]){
                            NSString *screenname = screen[@"name"];
                            
                            NSString *key;
                            key = [NSString stringWithFormat:@"%@_%@",kJA_screenHit,screenname.lowercaseString];
                            if ([self isHitCountValid:weightage.integerValue]) {
                                [ruleBookDictionary setObject:weightage forKey:key];
                                [triggers addObject:key];
                                [self setNumberIfNotExist:@0 forKey:key];
                            }
                        }
                    }
                    [ruleBookDictionaryArray addObject:ruleBookDictionary];
                }
                
                for (NSDictionary *set in scorebased){
                    ruleBookDictionary = [[NSMutableDictionary alloc] init];
                    [ruleBookDictionary setObject:@"scorebased" forKey:@"type"];
                    [ruleBookDictionary setObject:set[@"goalscore"] forKey:@"goalscore"];
                    [ruleBookDictionary setObject:set[@"criteriaid"] forKey:@"criteriaid"];
                    NSDictionary *session = set[@"session"];
                    NSNumber *hitcount = session[@"weightage"];
                    NSDictionary *sessionduration = session[@"sessionduration"];
                    if ([self isHitCountValid:hitcount.integerValue]) {
                        [ruleBookDictionary setObject:hitcount forKey:kJA_sessionHit];
                        [ruleBookDictionary setObject:sessionduration forKey:[NSString stringWithFormat:@"%@_duration",kJA_sessionHit]];
                        [self setNumberIfNotExist:@0 forKey:kJA_sessionHit];
                    }
                    
                    NSMutableArray *_anchors = [NSMutableArray new];
                    NSArray *events = [set valueForKeyPath:@"anchorpoint.eventconf.events"];
                    NSString *group = [set valueForKeyPath:@"anchorpoint.eventconf.group"];
                    for (NSDictionary *event in events){
                        NSString *name = event[@"name"];
                        NSString *key;
                        if (group){
                            key = [NSString stringWithFormat:@"%@_%@_%@",kJA_eventHit,group.lowercaseString,name.lowercaseString];
                        }else{
                            key = [NSString stringWithFormat:@"%@_%@",kJA_eventHit,name.lowercaseString];
                        }
                        
                        [_anchors addObject:key];
                    }
                    NSArray *screens = [set valueForKeyPath:@"anchorpoint.screenconf.screens"];
                    for (NSDictionary *screen in screens){
                        NSString *screenname = screen[@"name"];
                        NSString *key = [NSString stringWithFormat:@"%@_%@",kJA_screenHit,screenname.lowercaseString];
                        [_anchors addObject:key];
                    }
                    
                    [anchors addObjectsFromArray:_anchors];
                    [ruleBookDictionary setObject:_anchors forKey:@"anchorpoint"];
                    
                    for (NSDictionary *eventconf in set[@"eventconf"]){
                        NSNumber *weightage = eventconf[@"weightage"];
                        NSString *group = eventconf[@"group"];
                        for (NSDictionary *event in eventconf[@"events"]){
                            NSString *name = event[@"name"];
                            NSString *key;
                            if (group){
                                key = [NSString stringWithFormat:@"%@_%@_%@",kJA_eventHit,group.lowercaseString,name.lowercaseString];
                            }else{
                                key = [NSString stringWithFormat:@"%@_%@",kJA_eventHit,name.lowercaseString];
                            }
                            if ([self isHitCountValid:weightage.integerValue]) {
                                [ruleBookDictionary setObject:weightage forKey:key];
                                [triggers addObject:key];
                                [self setNumberIfNotExist:@0 forKey:key];
                            }
                        }
                    }
                    
                    for (NSDictionary *screenconf in set[@"screenconf"]){
                        NSNumber *weightage = screenconf[@"weightage"];
                        for (NSDictionary *screen in screenconf[@"screens"]){
                            NSString *screenname = screen[@"name"];
                            
                            NSString *key;
                            key = [NSString stringWithFormat:@"%@_%@",kJA_screenHit,screenname.lowercaseString];
                            if ([self isHitCountValid:weightage.integerValue]) {
                                [ruleBookDictionary setObject:weightage forKey:key];
                                [triggers addObject:key];
                                [self setNumberIfNotExist:@0 forKey:key];
                            }
                        }
                    }
                    [ruleBookDictionaryArray addObject:ruleBookDictionary];
                }
                //}
                
            });
        } @catch (NSException *exception) {
            IVLog(@"Exception occured while parsing rateus data - %@",exception);
        }
    });
}

-(BOOL) shouldUpdateData{
    
    if (!self.enableRateUsAndReview){
        return false;
    }
    
    NSNumber *numberOfAttempts = [ZAKeyChainWrapper numberForKey:kJA_autoReviewAttempt];
    NSNumber *impressionLimit = [ZAKeyChainWrapper numberForKey:kJA_autoReviewImpressionLimit];
    
    if (numberOfAttempts != nil && impressionLimit != nil){
        if (numberOfAttempts.intValue >= impressionLimit.intValue){
            return false;
        }
    }
    return true;
}

-(BOOL) isHitCountValid: (NSInteger) Integer {
    if (Integer)
    {
        NSNumber *number = [NSNumber numberWithInteger:Integer];
        if (number != nil) {
            return true;
        }
    }
    return false;
}


-(void) checkForRatingPop{
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.5 * NSEC_PER_SEC),self.zaSerialQueue, ^{
        if (!inProgress){
            inProgress = true;
            [self validateAndShowReviewPrompt:nil];
        }
    });
}

-(void) showStoreAlertOnFulFillingCriteria : (bool) status{
    _showStoreAlertOnFulFillingCriteria = status;
}

- (NSNumber*)numberForKey:(NSString *)key;{
    id hitcount = [engagementsAggrigateData valueForKey:key];
    if (hitcount){
        return (NSNumber*)hitcount;
    }else
        return @0;
    
}

- (void)setNumber:(NSNumber*)number forKey:(NSString *)key;{
    if (number && [engagementsAggrigateData objectForKey:key])
        [engagementsAggrigateData setValue:number forKey:key];
}

- (void)setNumberIfNotExist:(NSNumber*)number forKey:(NSString *)key;{
    if (number && ![engagementsAggrigateData objectForKey:key])
        [engagementsAggrigateData setValue:number forKey:key];
}

- (void)dealloc{
    [self saveAggrigatedEngagementsData];
    NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter];
    [notificationCenter removeObserver:self name:@"com.apticssdk.RateUsWindowClose" object:nil];
}

#if !TARGET_OS_WATCH
- (void) applicationDidEnterBackground:(NSNotification *)notification
{
    [self saveAggrigatedEngagementsData];
}

- (void) appWillTerminate:(NSNotification *)notification
{
    [self saveAggrigatedEngagementsData];
}

- (void) crashFoundNotification:(NSNotification *)notification{
    self.shouldPauseRateUsAndReview = true;
}

- (void) rateusNotification:(NSNotification *)notification{
    NSDictionary *object = (NSDictionary*)notification.object;
    if (object != nil){
        NSString *action = object[@"action"];
        if ([action isEqual:@"event"]){
            NSString *name = object[@"name"];
            NSString *_id = object[@"id"];
            if (_id != nil){
                [self incrementEventHitCountForEventId:_id];
            }
            if (name != nil){
                [self incrementEventHitCount:name];
            }
        }else if ([action isEqual:@"screen"]){
            NSString *name = object[@"name"];
            if (name != nil){
                [self incrementScreenHitCount:name];
            }
        }else if ([action isEqual:@"session"]){
            NSNumber *duration = object[@"duration"];
            if (duration != nil){
                [self incrementSessionHitCount:duration.integerValue];
            }
        }else if ([action isEqual:@"loaddata"]){
            [self loadRuleBook];
        }else{
            
        }
    }
}

#endif
@end

