import 'package:apptics_flutter/rateus/popup_action.dart';
import 'package:flutter/src/widgets/framework.dart';
import 'package:plugin_platform_interface/plugin_platform_interface.dart';

import 'apptics_flutter_method_channel.dart';
import 'apptics_flutter_util.dart';

abstract class AppticsFlutterPlatform extends PlatformInterface {
  /// Constructs a AppticsFlutterPlatform.
  AppticsFlutterPlatform() : super(token: _token);

  static final Object _token = Object();

  static AppticsFlutterPlatform _instance = MethodChannelAppticsFlutter();

  /// The default instance of [AppticsFlutterPlatform] to use.
  ///
  /// Defaults to [MethodChannelAppticsFlutter].
  static AppticsFlutterPlatform get instance => _instance;

  /// Platform-specific implementations should set this with their own
  /// platform-specific class that extends [AppticsFlutterPlatform] when
  /// they register themselves.
  static set instance(AppticsFlutterPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  Future<void> setCrashCustomProperty(Map<String, dynamic> properties) {
    throw UnimplementedError(
        '`setCrashCustomProperty` has not been implemented.');
  }

  Future<void> presentPrivacyReviewPopup() {
    throw UnimplementedError(
        '`presentPrivacyReviewPopup` has not been implemented.');
  }

  Future<void> openPrivacySettings() {
    throw UnimplementedError('`openPrivacySettings` has not been implemented.');
  }

  Future<void> setUser(String? userId) {
    throw UnimplementedError('`setUser` has not been implemented.');
  }

  Future<TrackingState?> getTrackingState() {
    throw UnimplementedError('`getTrackingState` has not been implemented.');
  }

  Future<void> setTrackingState(TrackingState status) {
    throw UnimplementedError('`setTrackingState` has not been implemented.');
  }

  Future<bool?> isUserLoggedIn() {
    throw UnimplementedError('`isUserLoggedIn` has not been implemented.');
  }

  Future<void> addEvent(
      String? event, String? group, Map<String, dynamic>? properties) {
    throw UnimplementedError('`addEvent` has not been implemented.');
  }

  Future<void> screenAttached(String screen_name) {
    throw UnimplementedError('`screenAttached` has not been implemented.');
  }

  Future<void> screenDetached(String screen_name) {
    throw UnimplementedError('`screenDetached` has not been implemented.');
  }

  Future<void> showLastSessionCrashedPopup() {
    throw UnimplementedError(
        '`showLastSessionCrashedPopup` has not been implemented.');
  }

  Future<void> setDefaultLanguage(String lang) {
    throw UnimplementedError('`setDefaultLanguage` has not been implemented.');
  }

  Future<void> checkAndUpdateAlert(BuildContext context) {
    throw UnimplementedError('`checkAndUpdateAlert` has not been implemented.');
  }

  Future<void> onClickUpdate() {
    throw UnimplementedError('`onClickUpdate` has not been implemented.');
  }

  Future<void> onClickReminder() {
    throw UnimplementedError('`onClickReminder` has not been implemented.');
  }

  Future<void> onClickIgnore() {
    throw UnimplementedError('`onClickIgnore` has not been implemented.');
  }

  Future<void> onClickNonSupportAlert() {
    throw UnimplementedError(
        '`onClickNonSupportAlert` has not been implemented.');
  }

  Future<void> onSendImpressionStatus(String updateId) {
    throw UnimplementedError(
        '`onSendImpressionStatus` has not been implemented.');
  }

  Future<Map<String, dynamic>?> getInAppUpdateData() {
    throw UnimplementedError('`getInAppUpdateData` has not been implemented.');
  }

  Future<int?> checkForRatingPop() {
    throw UnimplementedError('`checkForRatingPop` has not been implemented.');
  }
  Future<bool?> isAppticsFeedbackModuleAvailable() {
    throw UnimplementedError('`isAppticsFeedbackModuleAvailable` has not been implemented.');
  }

  Future<void> openFeedbackScreen() {
    throw UnimplementedError('`openFeedbackScreen` has not been implemented.');

  }

  int get daysBeforeShowingPopupAgain {
    throw UnimplementedError('`daysBeforeShowingPopupAgain` is not implemented');
  }

  int get maxTimesToShowPopup {
    throw UnimplementedError('`maxTimesToShowPopup` is not implemented');
  }

  bool get disableAutoPromptOnFulFillingCriteria {
    throw UnimplementedError('`disableAutoPromptOnFulFillingCriteria` is not implemented');
  }

  bool get showStoreAlertOnFulFillingCriteria {
    throw UnimplementedError('`showStoreAlertOnFulFillingCriteria` is not implemented');
  }

  bool get disableIfNotInstalledFromPlayStore {
    throw UnimplementedError('`disableIfNotInstalledFromPlayStore` is not implemented');
  }

  Future<void> setDaysBeforeShowingPopupAgain(int dayCount) async {
    throw UnimplementedError('`setDaysBeforeShowingPopupAgain` is not implemented');
  }

  Future<void> setMaxTimesToShowPopup(int maxTime) async {
    throw UnimplementedError('`setMaxTimesToShowPopup` is not implemented');
  }

  Future<void> setDisableAutoPromptOnFulFillingCriteria(bool isDisable) async {
    throw UnimplementedError('`setDisableAutoPromptOnFulFillingCriteria` is not implemented');
  }

  Future<void> setShowStoreAlertOnFulFillingCriteria(
      bool isShow) async {
    throw UnimplementedError(
        '`setShowStoreAlertOnFulFillingCriteria` is not implemented');
  }

  Future<void> setDisableIfNotInstalledFromPlayStore(
      bool ifNotInstalledFromPlayStore) async {
    throw UnimplementedError(
        '`setShowAndroidPlayCoreAlertOnFulFillingCriteria` is not implemented');
  }

  Future<void> sentRatingStats(int criteriaId, PopupAction popupAction)  async{
    throw UnimplementedError('`sentRatingStats` is not implemented');
  }

  Future<void> updateRatingShown() async{
    throw UnimplementedError('`updateRatingShown` is not implemented');
  }

  Future<void> openPlayStore()  async{
    throw UnimplementedError('`openPlayStore` is not implemented');
  }

  Future<void> hardReset() async {
    throw UnimplementedError('`hardReset` is not implemented');
  }

  Future<String?> getStringValue(String key, {
    bool coldFetch = false,
    bool fallbackWithOfflineValue = false,
  }) async {
    throw UnimplementedError('`getStringValue` is not implemented');
  }

  Future<void> setCustomConditionValue(String key, String value) async {
    throw UnimplementedError('`setCustomConditionValue` is not implemented');
  }  

}
