import 'package:apptics_flutter/appupdate/appupdate_util.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class AppticsInAppUpdateAlert {
  static Future show({
    required BuildContext context,
    required AppUpdateData appUpdateData,
    required void Function() onClickUpdate,
    required void Function() onClickReminder,
    required void Function() onClickIgnore,
  }) {
    AlertDialog child;
    final isDarkMode =
        MediaQuery.of(context).platformBrightness == Brightness.dark;
    bool isBackPress = false;

    if (appUpdateData.alertType == 0) {
      /// Apptics Native Alert
      List<Widget> widgetList = <Widget>[];
      final updateTextButton = TextButton(
        style: TextButton.styleFrom(
          textStyle: Theme.of(context).textTheme.labelLarge,
        ),
        child: Text(appUpdateData.updateNowText.toUpperCase()),
        onPressed: () {
          onClickUpdate();
          Navigator.of(context).pop();
        },
      );

      final remindMeTextButton = TextButton(
        style: TextButton.styleFrom(
          textStyle: Theme.of(context).textTheme.labelLarge,
        ),
        child: Text(appUpdateData.remindMeLaterText.toUpperCase()),
        onPressed: () {
          onClickReminder();
          Navigator.of(context).pop();
        },
      );
      final ignoreTextButton = TextButton(
        style: TextButton.styleFrom(
          textStyle: Theme.of(context).textTheme.labelLarge,
        ),
        child: Text(appUpdateData.neverAgainText.toUpperCase()),
        onPressed: () {
          onClickIgnore();
          Navigator.of(context).pop();
        },
      );

      widgetList.add(updateTextButton);
      if (appUpdateData.option == "1") {
        widgetList.add(remindMeTextButton);
        widgetList.add(ignoreTextButton);
        isBackPress = true;
      } else if (appUpdateData.option == "2") {
        widgetList.add(remindMeTextButton);
      }
      child = AlertDialog(
        title: Text(appUpdateData.featureTitle,
            style: isDarkMode
                ? const TextStyle(color: Colors.white)
                : const TextStyle(color: Colors.black)),
        backgroundColor: isDarkMode ? Colors.black : Colors.white,
        content: Text(appUpdateData.features),
        contentTextStyle: isDarkMode
            ? const TextStyle(color: Colors.white)
            : const TextStyle(color: Colors.black),
        actions: widgetList,
      );
    } else if (appUpdateData.alertType == 1) {
      /// Apptics Custom Alert
      child = AlertDialog(
        contentPadding: const EdgeInsets.all(0),
        backgroundColor: isDarkMode ? Colors.black : Colors.white,
        contentTextStyle: isDarkMode
            ? const TextStyle(color: Colors.white)
            : const TextStyle(color: Colors.black),
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(15.0),
        ),
        content: buildPopUp(context, appUpdateData, isDarkMode, () {
          onClickUpdate();
        }, () {
          onClickReminder();
        }, () {
          onClickIgnore();
        }, (bool isBack) {
          isBackPress = isBack;
        }),
      );
    } else if (appUpdateData.alertType == 2) {
      // Android play store app update
      child = AlertDialog(
        contentPadding: const EdgeInsets.all(0),
        backgroundColor: isDarkMode ? Colors.black : Colors.white,
        contentTextStyle: isDarkMode
            ? const TextStyle(color: Colors.white)
            : const TextStyle(color: Colors.black),
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(15.0),
        ),
        content: buildPopUp(context, appUpdateData, isDarkMode, () {
          onClickUpdate();
        }, () {
          onClickReminder();
        }, () {
          onClickIgnore();
        }, (bool isBack) {
          isBackPress = isBack;
        }),
      );
    } else {
      child = AlertDialog(content: Container());
    }

    return showDialog(
        context: context,
        barrierDismissible: false,
        useRootNavigator: false,
        builder: (context) {
          return WillPopScope(
              child: child,
              onWillPop: () async {
                if (isBackPress) {
                  onClickIgnore();
                } else {
                  SystemNavigator.pop();
                }
                return isBackPress;
              });
        });
  }

  static Widget buildPopUp(
      BuildContext context,
      AppUpdateData appUpdateData,
      bool isDarkMode,
      void Function() onClickUpdate,
      void Function() onClickReminder,
      void Function() onClickIgnore,
      void Function(bool) checkIsBack) {
    String imageSource;
    if (isDarkMode) {
      imageSource = 'assets/images-dark/ic_version_alert.png';
    } else {
      imageSource = 'assets/images/ic_version_alert.png';
    }

    List<Widget> actionWidgetList = <Widget>[
      ElevatedButton(
        onPressed: () {
          onClickUpdate();
          Navigator.pop(context);
        },
        style: ElevatedButton.styleFrom(
            padding: const EdgeInsets.symmetric(horizontal: 60.0),
            shape: const StadiumBorder()),
        child: Text(appUpdateData.updateNowText),
      ),
      const SizedBox(height: 15.0)
    ];
    final remindMeTextButton = TextButton(
      onPressed: () {
        onClickReminder();
        Navigator.pop(context);
      },
      style: TextButton.styleFrom(),
      child: Text(appUpdateData.remindMeLaterText),
    );
    final ignoreTextButton = TextButton(
        onPressed: () {
          onClickIgnore();
          Navigator.pop(context);
        },
        style: TextButton.styleFrom(),
        child: Text(appUpdateData.neverAgainText));
    checkIsBack(false);
    if (appUpdateData.option == "1") {
      actionWidgetList.add(remindMeTextButton);
      actionWidgetList.add(ignoreTextButton);
      checkIsBack(true);
    } else if (appUpdateData.option == "2") {
      actionWidgetList.add(remindMeTextButton);
    }
    return SizedBox(
      width: double.infinity,
      child: SingleChildScrollView(
        child: Column(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Container(
              padding: const EdgeInsets.all(16),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  Image(
                      image:
                          AssetImage(imageSource, package: 'apptics_flutter'),
                      fit: BoxFit.cover),
                  const SizedBox(height: 25),
                  Text(
                    appUpdateData.featureTitle,
                    textAlign: TextAlign.center,
                    style: const TextStyle(
                        fontWeight: FontWeight.bold, fontSize: 18.0),
                  ),
                  const SizedBox(height: 25),
                  SingleChildScrollView(
                    child: Text(
                      appUpdateData.features,
                      textAlign: TextAlign.center,
                      style: const TextStyle(fontSize: 16.0),
                    ),
                  ),
                ],
              ),
            ),
            const SizedBox(height: 50),
            SizedBox(
              width: double.infinity,
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                children: actionWidgetList,
              ),
            )
          ],
        ),
      ),
    );
  }

  static Future showNonSupportAlert({
    required BuildContext context,
    required AppticsAppUpdateNotSupported appUpdateData,
    required void Function() onClick,
  }) {
    List<Widget>? actionWidget;

    if (appUpdateData.alertType == 1) {
      actionWidget = [
        TextButton(
          style: TextButton.styleFrom(
            textStyle: Theme.of(context).textTheme.labelLarge,
          ),
          child: Text(appUpdateData.continueBtTxt),
          onPressed: () {
            onClick();
            Navigator.of(context).pop();
          },
        )
      ];
    } else {
      actionWidget = null;
    }
    return showDialog(
        context: context,
        barrierDismissible: false,
        useRootNavigator: false,
        builder: (context) {
          return WillPopScope(
              child: AlertDialog(
                title: Text(appUpdateData.title),
                content: SingleChildScrollView(
                    child: Text(appUpdateData.description)),
                actions: actionWidget,
              ),
              onWillPop: () async {
                SystemNavigator.pop();
                return false;
              });
        });
  }
}
