import '../apptics_flutter_platform_interface.dart';

class AppticsRemoteConfig {
  static AppticsRemoteConfig? _instance;
  static AppticsRemoteConfig get instance {
    _instance ??= AppticsRemoteConfig();
    return _instance!;
  }

  Future<void> hardReset() async {
    return await AppticsFlutterPlatform.instance.hardReset();
  }

  /// Fetch param value.
  /// By default, param value is fetched from server. If offline value is required, set true to [fallbackWithOfflineValue] param. Only for [Android].
  ///
  /// @param key [String] param key.
  /// @param coldFetch [bool] parameter with a default value of false. If enabled, it fetches current data from the server; otherwise, it retrieves data from the cache.
  /// @param fallbackWithOfflineValue [Boolean] optional param. False if offline value is not required. (Only for [Android])
  Future<String?> getStringValue(String key, {
    bool coldFetch = false,
    bool fallbackWithOfflineValue = false,
  }) async {
    return await AppticsFlutterPlatform.instance.getStringValue(key, coldFetch:coldFetch, fallbackWithOfflineValue:fallbackWithOfflineValue);
  }

  /// Set custom condition key and value.
  /// @param key [String] key.
  /// @param value [String] Value for the condition.
  Future<void> setCustomConditionValue(String key, String value) async {
    return await AppticsFlutterPlatform.instance.setCustomConditionValue(key, value);
  }

}
